var ask, build, buildBrowser, buildNode, bundle, clean, coffee, commands, compile, defaults, docs, exec, fillDefaults, fs, glob, help, ifInstalled, knownTarget, loadInfo, makeDir, nodeunit, packageFile, path, publish, readline, rimraf, rmrf, run, scripts, stitch, test, tryExec, tryWithFile, uglify, ver, version, watch, watchFiles, which, withFiles;
coffee = require('coffee-script');
fs = require('fs');
glob = require('glob');
nodeunit = require('nodeunit');
path = require('path');
readline = require('readline');
rimraf = require('rimraf');
stitch = require('stitch');
uglify = require('uglify-js');
which = require('which');
exec = require('child_process').exec;
defaults = {
  browser: {
    enabled: true,
    paths: null,
    prefix: '',
    rootModule: null
  },
  copyrightFrom: (new Date).getFullYear(),
  license: '',
  hooks: {
    tokenize: null,
    parse: null,
    compile: null
  },
  paths: null
};
fillDefaults = function(obj, defaults) {
  var key, result, value;
  result = {};
  for (key in defaults) {
    value = defaults[key];
    result[key] = obj[key] != null ? (typeof obj[key] === 'object') && (typeof value === 'object') ? fillDefaults(obj[key], value) : obj[key] : value;
  }
  return result;
};
loadInfo = function() {
  var copyright, currentYear, hook, info, json, key, license, module, overrides, package, paths, pth, value, _, _base, _base2, _i, _len, _ref, _ref2, _ref3, _ref4, _ref5;
  try {
    json = fs.readFileSync('beans.json');
  } catch (e) {
    json = '{}';
  }
  overrides = JSON.parse(json);
  info = fillDefaults(overrides, defaults);
  _ref = info.hooks;
  for (hook in _ref) {
    module = _ref[hook];
    if (module != null) {
      info.hooks[hook] = require(path.resolve(module));
    }
  }
  if (overrides.paths == null) {
    overrides.paths = {
      src: 'lib'
    };
  }
  paths = {};
  _ref2 = overrides.paths;
  for (key in _ref2) {
    value = _ref2[key];
    paths[path.resolve(key)] = path.resolve(value);
  }
  info.paths = paths;
  if ((_ref3 = (_base = info.browser).paths) == null) {
    _base.paths = (function() {
      var _ref4, _results;
      _ref4 = info.paths;
      _results = [];
      for (_ in _ref4) {
        pth = _ref4[_];
        _results.push(pth);
      }
      return _results;
    })();
  }
  package = JSON.parse(fs.readFileSync('package.json'));
  _ref4 = ['author', 'name', 'description', 'version'];
  for (_i = 0, _len = _ref4.length; _i < _len; _i++) {
    key = _ref4[_i];
    if (package[key] == null) {
      throw new Error("Section \"" + key + "\" required in package.json.");
    }
    info[key] = package[key];
  }
  if ((_ref5 = (_base2 = info.browser).rootModule) == null) {
    _base2.rootModule = info.name;
  }
  info.browser.name = info.browser.prefix + info.name;
  currentYear = (new Date).getFullYear();
  if (currentYear > info.copyrightFrom) {
    copyright = info.copyrightFrom + '-' + currentYear;
  } else {
    copyright = currentYear;
  }
  if (info.license !== '') {
    license = "Released under the " + info.license;
  } else {
    license = 'Contact author for licensing information';
  }
  info.headerComment = "/**\n * " + info.browser.name + " " + info.version + " (browser bundle)\n * " + info.description + "\n *\n * Copyright (c) " + copyright + " " + info.author + "\n * " + license + "\n */\n";
  info.header = "this['" + info.name + "'] = (function(){";
  info.footer = "var module = this.require('" + info.browser.rootModule + "');\nmodule.version = '" + info.version + "';\nreturn module; }).call({});";
  return info;
};
ifInstalled = function(executable, fn) {
  return which(executable, function(err) {
    if (!err) {
      return fn();
    }
    return console.log("This task needs \"" + executable + "\" to be installed and in PATH.");
  });
};
tryExec = function(executable, args, fn) {
  return ifInstalled(executable, function() {
    var proc;
    proc = exec(executable + ' ' + args, function(err) {
      if (err) {
        throw err;
      }
      return typeof fn === "function" ? fn() : void 0;
    });
    return proc.stdout.on('data', function(data) {
      return process.stdout.write(data.toString());
    });
  });
};
rmrf = function(path, fn) {
  return rimraf(path, function(err) {
    if (err) {
      throw err;
    }
    return typeof fn === "function" ? fn() : void 0;
  });
};
makeDir = function(dir) {
  var missing, _i, _len, _results;
  missing = [];
  dir = path.resolve(dir);
  while (!path.existsSync(dir)) {
    missing.unshift(dir);
    dir = path.dirname(dir);
  }
  _results = [];
  for (_i = 0, _len = missing.length; _i < _len; _i++) {
    dir = missing[_i];
    _results.push(fs.mkdirSync(dir, 0755));
  }
  return _results;
};
withFiles = function(pattern, fn) {
  var files;
  files = glob.globSync(pattern);
  if (files.length > 0) {
    return fn(files);
  }
};
ask = function(question, answers, fn) {
  var stdin, stdout;
  stdin = process.stdin, stdout = process.stdout;
  stdout.write(question + ' (' + answers.join('/') + ') ');
  stdin.resume();
  return stdin.on('data', function(answer) {
    answer = answer.toString().trim();
    if (answers.indexOf(answer) !== -1) {
      stdin.pause();
      return fn(answer);
    } else {
      return stdout.write('Please answer with one of: (' + answers.join('/') + ') ');
    }
  });
};
knownTarget = function(command, target, targets) {
  if (targets.indexOf(target) === -1) {
    console.log("Unknown " + command + " target \"" + target + "\".");
    console.log("Try one of: " + (targets.join(', ')) + ".");
    return false;
  }
  return true;
};
watchFiles = function(files, fn) {
  var file, _i, _len, _results;
  _results = [];
  for (_i = 0, _len = files.length; _i < _len; _i++) {
    file = files[_i];
    _results.push((function(file) {
      return fs.watchFile(file, {
        persistent: true,
        interval: 500
      }, function(curr, prev) {
        if (curr.mtime.getTime() !== prev.mtime.getTime()) {
          return fn(file);
        }
      });
    })(file));
  }
  return _results;
};
tryWithFile = function(file, event, fn) {
  try {
    return fn();
  } catch (err) {
    err.message = "When " + event + " " + file + ", " + err.message;
    throw err;
  }
};
compile = function(info, file, sourcePath, targetPath) {
  var source, src, target, ts;
  file = fs.realpathSync(file);
  src = fs.readFileSync(file, 'utf8');
  source = file.substr(sourcePath.length);
  target = targetPath + source.replace(/.coffee$/, '.js');
  tryWithFile(file, 'tokenizing', function() {
    return src = coffee.tokens(src);
  });
  if (info.hooks.tokenize != null) {
    src = info.hooks.tokenize(target, src);
  }
  tryWithFile(file, 'parsing', function() {
    return src = coffee.nodes(src);
  });
  if (info.hooks.parse != null) {
    src = info.hooks.parse(target, src);
  }
  tryWithFile(file, 'compiling', function() {
    return src = src.compile({
      bare: true
    });
  });
  if (info.hooks.compile != null) {
    src = info.hooks.compile(target, src);
  }
  makeDir(path.dirname(target));
  fs.writeFileSync(target, src);
  ts = (new Date()).toLocaleTimeString();
  source = sourcePath.substr(path.resolve('.').length + 1) + source;
  return console.log(ts + ' - compiled ' + source);
};
buildNode = function(info, watch, fn) {
  var compiled, count, sourcePath, targetPath, _, _ref, _results;
  count = 0;
  for (_ in info.paths) {
    count++;
  }
  compiled = 0;
  _ref = info.paths;
  _results = [];
  for (sourcePath in _ref) {
    targetPath = _ref[sourcePath];
    _results.push((function(sourcePath, targetPath) {
      return withFiles(path.join(sourcePath, '**/*.coffee'), function(files) {
        var file, _i, _len;
        for (_i = 0, _len = files.length; _i < _len; _i++) {
          file = files[_i];
          compile(info, file, sourcePath, targetPath);
        }
        if (watch) {
          watchFiles(files, function(file) {
            try {
              return compile(info, file, sourcePath, targetPath);
            } catch (err) {
              return console.log(err.stack);
            }
          });
        }
        if (++compiled === count) {
          return typeof fn === "function" ? fn() : void 0;
        }
      });
    })(sourcePath, targetPath));
  }
  return _results;
};
bundle = function(info) {
  var path, paths;
  paths = (function() {
    var _i, _len, _ref, _results;
    _ref = info.browser.paths;
    _results = [];
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      path = _ref[_i];
      _results.push(fs.realpathSync(path));
    }
    return _results;
  })();
  return stitch.createPackage({
    paths: paths
  }).compile(function(err, src) {
    var dir, fname;
    if (err) {
      throw err;
    }
    makeDir('build');
    makeDir('build/' + info.version);
    dir = fs.realpathSync('build/' + info.version);
    fname = dir + '/' + info.browser.name;
    src = info.header + src + info.footer;
    fs.writeFileSync(fname + '.js', info.headerComment + src);
    fs.writeFileSync(fname + '.min.js', info.headerComment + uglify(src));
    try {
      fs.unlinkSync('build/edge');
    } catch (_e) {}
    return fs.symlinkSync(dir + '/', 'build/edge');
  });
};
buildBrowser = function(info, watch) {
  var paths, pth;
  bundle(info);
  if (watch) {
    paths = (function() {
      var _i, _len, _ref, _results;
      _ref = info.browser.paths;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        pth = _ref[_i];
        _results.push(path.join(pth, '**/*.{coffee,js}'));
      }
      return _results;
    })();
    return watchFiles(glob.globSync("{" + (paths.join()) + "}"), function() {
      return bundle(info);
    });
  }
};
build = function(fn) {
  var info;
  info = loadInfo();
  return buildNode(info, false, function() {
    if (info.browser.enabled) {
      buildBrowser(info);
    }
    return typeof fn === "function" ? fn() : void 0;
  });
};
clean = function(target) {
  var dir, info, paths, pth, _, _i, _j, _len, _len2, _ref, _ref2;
  if (target == null) {
    target = 'all';
  }
  if (!knownTarget('clean', target, ['build', 'docs', 'all'])) {
    return;
  }
  info = loadInfo();
  paths = (function() {
    var _ref, _results;
    _ref = info.paths;
    _results = [];
    for (_ in _ref) {
      pth = _ref[_];
      _results.push(pth);
    }
    return _results;
  })();
  switch (target) {
    case 'build':
      _ref = paths.concat('build');
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        dir = _ref[_i];
        rmrf(dir);
      }
      break;
    case 'docs':
      rmrf('docs');
      break;
    case 'all':
      _ref2 = paths.concat('build', 'docs');
      for (_j = 0, _len2 = _ref2.length; _j < _len2; _j++) {
        dir = _ref2[_j];
        rmrf(dir);
      }
  }
};
docs = function() {
  var info, paths, pth;
  info = loadInfo();
  paths = (function() {
    var _results;
    _results = [];
    for (pth in info.paths) {
      _results.push(path.join(pth, '**/*.coffee'));
    }
    return _results;
  })();
  return withFiles("{" + (paths.join()) + "}", function(files) {
    return tryExec('docco', '"' + files.join('" "') + '"');
  });
};
help = function() {
  var command, name, _results;
  _results = [];
  for (name in commands) {
    command = commands[name];
    _results.push(console.log("beans " + name + "\t" + command[1]));
  }
  return _results;
};
publish = function() {
  return build(function() {
    return tryExec('npm', 'publish');
  });
};
scripts = function() {
  return ask('This will modify package.json. Proceed?', ['y', 'n'], function(answer) {
    var deps, package, script, _i, _len, _ref;
    if (answer === 'y') {
      package = JSON.parse(fs.readFileSync('package.json'));
      deps = package.devDependencies || (package.devDependencies = {});
      deps.beans || (deps.beans = '~' + ver);
      scripts = package.scripts || (package.scripts = {});
      _ref = ['build', 'clean', 'docs', 'test', 'watch'];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        script = _ref[_i];
        scripts[script] = 'beans ' + script;
      }
      scripts.prepublish = 'beans build';
      return fs.writeFileSync('package.json', JSON.stringify(package, null, 2) + "\n");
    }
  });
};
test = function() {
  return build(function() {
    return withFiles('test/**/*.test.coffee', function(files) {
      return nodeunit.reporters["default"].run(files);
    });
  });
};
packageFile = path.resolve(__dirname, '../package.json');
ver = JSON.parse(fs.readFileSync(packageFile)).version;
version = function() {
  return console.log('Beans ' + ver);
};
watch = function() {
  var info;
  info = loadInfo();
  return buildNode(info, true, function() {
    if (info.browser.enabled) {
      return buildBrowser(info, true);
    }
  });
};
commands = {
  build: [build, 'Compile CoffeScript source for enabled targets.'],
  clean: [clean, 'Remove generated directories and tidy things up.'],
  docs: [docs, 'Generate documentation files using Docco.'],
  help: [help, 'Display help (this text).'],
  publish: [publish, 'Build everything and run npm publish.'],
  scripts: [scripts, 'Register beans in package.json scripts.'],
  test: [test, 'Build everything and run tests using nodeunit.'],
  version: [version, 'Display current Beans version.'],
  watch: [watch, 'Build everything once, then watch for changes.']
};
run = function() {
  var args, _ref;
  args = process.argv.slice(2);
  if (args.length === 0) {
    return help();
  } else if (commands[args[0]] != null) {
    return (_ref = commands[args[0]])[0].apply(_ref, args.slice(1));
  } else {
    return console.log("Don't know how to \"" + args[0] + "\".");
  }
};
module.exports = {
  commands: commands,
  run: run,
  version: ver
};